==> japhar-0.07.tar.gz: japhar-0.07/lib/libnative/java.io/file.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   file.c -- native methods pertaining to java.io.File
   Created: Chris Toshok <toshok@hungry.com>, 14-Aug-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
#include "config.h"

#include "jni.h"
#include "exceptions.h" /* For throw_Exception() */
#include "compat.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <assert.h>

static char *
get_path(JNIEnv *env,
	 jobject obj)
{
  jfieldID path_id;
  jstring path;
  const jbyte *filename;
  char *result;
  jclass clazz = (*env)->GetObjectClass(env, obj);

  path_id = (*env)->GetFieldID(env, clazz, "path", "Ljava/lang/String;");
  if (!path_id) return NULL;

  path = (*env)->GetObjectField(env, obj, path_id);
  if (!path) return NULL;

  filename = (*env)->GetStringUTFChars(env, path, NULL);

  result = strdup((char*)filename);

  (*env)->ReleaseStringUTFChars(env, path, filename);

  return result;
}

static int
stat_file(JNIEnv *env,
	  jobject obj,
	  struct stat *buf)
{
  int result;
  char *path = get_path(env, obj);

  result = stat((char*)path, buf);

  free(path);

  return result;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_exists0(JNIEnv *env,
			  jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return JNI_FALSE;
  else
    return JNI_TRUE;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_canWrite0(JNIEnv *env,
			    jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return JNI_FALSE;
  else
    return (buf.st_mode & S_IWUSR) != 0;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_canRead0(JNIEnv *env,
			   jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return JNI_FALSE;
  else
    return (buf.st_mode & S_IRUSR) != 0;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_isFile0(JNIEnv *env,
			  jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return JNI_FALSE;
  else
    return S_ISREG(buf.st_mode) != 0;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_isDirectory0(JNIEnv *env,
			       jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return JNI_FALSE;
  else
    return S_ISDIR(buf.st_mode) != 0;
}

JNIEXPORT jlong JNICALL
Java_java_io_File_lastModified0(JNIEnv *env,
				jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return 0;
  else
    return (jlong)buf.st_mtime;
}

JNIEXPORT jlong JNICALL
Java_java_io_File_length0(JNIEnv *env,
			  jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return 0;
  else
    return (jlong)buf.st_size;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_mkdir0(JNIEnv *env,
			 jobject obj)
{
  char *path = get_path(env, obj);
  jboolean result;

  result = (mkdir(path, 0777) == 0);

  free(path);
  return result;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_renameTo0(JNIEnv *env,
			    jobject obj,
			    jobject name)
{
  char *path = get_path(env, obj);
  const jbyte* name_bytes;
  jboolean result;

  name_bytes = (*env)->GetStringUTFChars(env, name, NULL);

  result = (rename(path, (char*)name_bytes) == 0);

  free(path);

  (*env)->ReleaseStringUTFChars(env, name, name_bytes);

  return result;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_delete0(JNIEnv *env,
			  jobject obj)
{
  char *path = get_path(env, obj);
  jboolean result;

  result = (unlink(path) == 0);

  free(path);

  return result;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_rmdir0(JNIEnv *env,
			 jobject obj)
{
  char *path = get_path(env, obj);
  jboolean result;

  result = (rmdir(path) == 0);

  free(path);

  return result;
}

static int
scandir_select(struct dirent *entry)
{
  if (!strcmp(entry->d_name, ".")
      || !strcmp(entry->d_name, ".."))
    return 0;
  else
    return 1;
}

JNIEXPORT jobjectArray JNICALL
Java_java_io_File_list0(JNIEnv *env,
			jobject obj)
{
  char *path = get_path(env, obj);
  struct dirent **entries;
  int num_entries;
  jobjectArray elements;
  jclass string_class = (*env)->FindClass(env, "java/lang/String");
  int i;

  num_entries = scandir(path, &entries, scandir_select, NULL);

  if (num_entries == -1)
    {
      throw_Exception(env, "java/lang/OutOfMemory", "In java.io.File.list0");
      return NULL;
    }

  elements = (*env)->NewObjectArray(env, num_entries, string_class,
				    NULL);
  elements = (*env)->NewGlobalRef(env, elements);

  for (i = 0; i < num_entries; i ++)
    {
      jobject element = (*env)->NewStringUTF(env, entries[i]->d_name);

      element = (*env)->NewGlobalRef(env, element);

      (*env)->SetObjectArrayElement(env, elements, i, element);
    }

  free(entries);

  return elements;
}

/*
 * Follow links and translate relative directory references.
 */
JNIEXPORT jstring JNICALL
Java_java_io_File_canonPath(JNIEnv *env,
			    jobject obj,
			    jobject path)
{
  (*env)->FatalError(env, "java.lang.File.canonPath0() unimplemented");
  return NULL;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_isAbsolute(JNIEnv *env,
			     jobject obj)
{
  jclass clazz;
  jfieldID separatorChar_id;
  jchar separator;
  jboolean absolute = JNI_FALSE;
  char *path;

  assert(NULL != env);
  assert(NULL != obj);

  path = get_path(env, obj);
  assert(NULL != path);

  clazz = (*env)->GetObjectClass(env, obj);
  assert(NULL != clazz);

  separatorChar_id = (*env)->GetStaticFieldID(env, clazz, "separatorChar", "C");
  assert(NULL != separatorChar_id);

  separator = (*env)->GetStaticCharField(env, clazz, separatorChar_id);
  /* XXX This assert triggers on SPARC/Solaris [pere 1998-05-08] */
  assert(0 != separator);

  absolute = path[0] == (separator & 0xff); /* XXX only handle ISO 8859/1 */

  /*
   * Handle MSDOS C:\full\path
   */
  if ( ! absolute && ('\\' == (separator & 0xff)) &&
       2 >= strlen(path) && ':' == path[1] && isalpha(path[0]) )
    absolute = JNI_TRUE;

  free(path);
  return absolute;
}

==> japhar-0.08.tar.gz: japhar-0.08/lib/libnative/java.io/file.c <==
/* -*- Mode: C; c-file-style: "gnu" -*-
   file.c -- native methods pertaining to java.io.File
   Created: Chris Toshok <toshok@hungry.com>, 14-Aug-1997
 */
/*
  This file is part of Japhar, the GNU Virtual Machine for Java Bytecodes.
  Japhar is a project of The Hungry Programmers, GNU, and OryxSoft.

  Copyright (C) 1997, 1998, 1999 The Hungry Programmers

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "jni.h"
#include "exceptions.h" /* For throw_Exception() */
#include "compat.h"
#include "common.h"

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <assert.h>

#ifdef HAVE_UNISTD_H
#  include <unistd.h>
#endif
#ifdef HAVE_DIRENT_H
#  include <dirent.h>
#elif defined( HAVE_DIRECT_H )
#  include <direct.h>
#endif

/**
 * WARNING: returned pointer must be freed by caller
 */
static char *
jstring2charptr(JNIEnv *env, jstring jstr)
{
  char *str_copy;
  const jbyte *str_chars;
  int str_length;
  str_chars = (*env)->GetStringUTFChars(env, jstr, NULL);
  str_length = (*env)->GetStringUTFLength(env, jstr) + 1;
  str_copy = (char*) malloc(str_length * sizeof(char));
  strncpy(str_copy, (char*)str_chars, str_length);
  (*env)->ReleaseStringUTFChars(env, jstr, str_chars);
  str_copy[str_length-1] = 0; /* Make sure the string is zero terminated */
  return str_copy;
}

/* JDK1.1 */
JNIEXPORT jboolean JNICALL
Java_java_io_File_exists0(JNIEnv *env,
			  jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return JNI_FALSE;
  else
    return JNI_TRUE;
}

/* JDK1.1 */
JNIEXPORT jboolean JNICALL
Java_java_io_File_canWrite0(JNIEnv *env,
			    jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return JNI_FALSE;
  else
#ifdef S_IWUSR
    return (buf.st_mode & S_IWUSR) != 0;
#elif defined( _S_IWRITE)
  return (buf.st_mode & _S_IWRITE) != 0;
#endif
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_canRead0(JNIEnv *env,
			   jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return JNI_FALSE;
  else
#ifdef S_IRUSR
    return (buf.st_mode & S_IRUSR) != 0;
#elif defined( _S_IREAD )
  return (buf.st_mode & _S_IREAD) != 0;
#endif
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_isFile0(JNIEnv *env,
			  jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return JNI_FALSE;
  else
#ifdef S_ISREG
    return S_ISREG(buf.st_mode) != 0;
#elif defined _S_IFREG
  return (buf.st_mode & _S_IFREG) != 0;
#endif
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_isDirectory0(JNIEnv *env,
			       jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return JNI_FALSE;
  else
#ifdef S_ISDIR
    return S_ISDIR(buf.st_mode) != 0;
#elif defined( _S_IFDIR )
  return (buf.st_mode & _S_IFDIR) != 0;
#endif
}

JNIEXPORT jlong JNICALL
Java_java_io_File_lastModified0(JNIEnv *env,
				jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return 0;
  else
    return (jlong)buf.st_mtime;
}

JNIEXPORT jlong JNICALL
Java_java_io_File_length0(JNIEnv *env,
			  jobject obj)
{
  struct stat buf;

  if (stat_file(env, obj, &buf) == -1)
    return 0;
  else
    return (jlong)buf.st_size;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_mkdir0(JNIEnv *env,
			 jobject obj)
{
  char *path = get_file_path(env, obj);
  jboolean result;

#ifdef MKDIR_TAKES_ONE_ARG
  /* windows defines this.  no need to check for it in configure.in */
  result = (mkdir(path) == 0);
#else
  result = (mkdir(path, 0777) == 0);
#endif

  free(path);
  return result;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_renameTo0(JNIEnv *env,
			    jobject obj,
			    jobject name)
{
  char *path = get_file_path(env, obj);
  const jbyte* name_bytes;
  jboolean result;

  name_bytes = (*env)->GetStringUTFChars(env, name, NULL);

  result = (rename(path, (char*)name_bytes) == 0);

  free(path);

  (*env)->ReleaseStringUTFChars(env, name, name_bytes);

  return result;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_delete0(JNIEnv *env,
			  jobject obj)
{
  char *path = get_file_path(env, obj);
  jboolean result;

  result = (unlink(path) == 0);

  free(path);

  return result;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_rmdir0(JNIEnv *env,
			 jobject obj)
{
  char *path = get_file_path(env, obj);
  jboolean result;

  result = (rmdir(path) == 0);

  free(path);

  return result;
}

static int
scandir_select(struct dirent *entry)
{
  if (!strcmp(entry->d_name, ".")
      || !strcmp(entry->d_name, ".."))
    return 0;
  else
    return 1;
}

JNIEXPORT jobjectArray JNICALL
Java_java_io_File_list0(JNIEnv *env,
			jobject obj)
{
  char *path = get_file_path(env, obj);
  struct dirent **entries;
  int num_entries;
  jobjectArray elements;
  jclass string_class = (*env)->FindClass(env, "java/lang/String");
  int i;

  num_entries = scandir(path, &entries, scandir_select, NULL);

  if (num_entries == -1)
    {
      throw_Exception(env, "java/lang/OutOfMemory", "In java.io.File.list0");
      return NULL;
    }

  elements = (*env)->NewObjectArray(env, num_entries, string_class,
				    NULL);
  elements = (*env)->NewGlobalRef(env, elements);

  for (i = 0; i < num_entries; i ++)
    {
      jobject element = (*env)->NewStringUTF(env, entries[i]->d_name);

      element = (*env)->NewGlobalRef(env, element);

      (*env)->SetObjectArrayElement(env, elements, i, element);
    }

  free(entries);

  return elements;
}

/* JDK1.1 */
JNIEXPORT jstring JNICALL
Java_java_io_File_canonPath(JNIEnv *env,
			    jobject obj,
			    jobject path)
{
  char *path_str = jstring2charptr(env, path);
  jstring ret_val;

#ifdef HAVE_RESOLVEPATH
  {
    char resulting_path[ PATH_MAX + 1 ];
    int len;

    len = resolvepath(path_str, resulting_path, sizeof(resulting_path));

    resulting_path[ len ] = 0;

    ret_val = (*env)->NewStringUTF(env, resulting_path);
  }
#else /* punt and don't transform the name at all */
  ret_val = path;
#endif

  free(path_str);

  return path;
}

JNIEXPORT jboolean JNICALL
Java_java_io_File_isAbsolute(JNIEnv *env,
			     jobject obj)
{
  jclass clazz;
  jfieldID separatorChar_id;
  jchar separator;
  jboolean absolute = JNI_FALSE;
  char *path;

  assert(NULL != env);
  assert(NULL != obj);

  path = get_file_path(env, obj);
  assert(NULL != path);

  clazz = (*env)->GetObjectClass(env, obj);
  assert(NULL != clazz);

  separatorChar_id = (*env)->GetStaticFieldID(env, clazz, "separatorChar", "C");
  assert(NULL != separatorChar_id);

  separator = (*env)->GetStaticCharField(env, clazz, separatorChar_id);
  /* XXX This assert triggers on SPARC/Solaris [pere 1998-05-08] */
  assert(0 != separator);

  absolute = path[0] == (separator & 0xff); /* XXX only handle ISO 8859/1 */

  /*
   * Handle MSDOS C:\full\path
   */
  if ( ! absolute && ('\\' == (separator & 0xff)) &&
       2 >= strlen(path) && ':' == path[1] && isalpha(path[0]) )
    absolute = JNI_TRUE;

  free(path);
  return absolute;
}
